<?php

class BaikalsrClient
{
    private static $apiUrlPrefixTest = "https://test-api.baikalsr.ru/";
    
    private static $apiUrlPrefix = "https://api.baikalsr.ru/";

    public static $settings;

    public function __construct ()
    {
        self::$settings = $this->getSettings ();
    }

    public function getSettings ()
    {
        $db = JFactory::getDbo ();

        $query = $db->getQuery (true);

        $query->select (array ('*'))
              ->from ($db->quoteName ('#__jshopping_baikalsr_settings', 's')) ;

        $db->setQuery ($query);

        $results = $db->loadAssocList ();

        return array_combine (
            array_column ($results, 'name'),
            array_column ($results, 'value')
        );
    }

    public function saveSettings ($settings)
    {
        $db = JFactory::getDbo ();

        foreach ($settings as $key => $value)
        {
            $fields = [ 
                $db->quoteName ('value') . ' = ' . $db->quote ($value)
            ];
            
            $condition = [
                $db->quoteName ('name') . ' = ' . $db->quote ($key)
            ];

            $query = $db->getQuery(true);
            $query->update ($db->quoteName ('#__jshopping_baikalsr_settings'))->set ($fields)->where ($condition);

            $db->setQuery ($query);
            $db->execute ();
        }
    } 

    private static function apiGetRequest ($apiUrl)
    {
        if (! (int) self::$settings ['TestMode'])
        {
            $apiKey = self::$settings ['ApiKey'];
            $apiUrl = self::$apiUrlPrefix . $apiUrl;
        }
        else
        {
            $apiKey = self::$settings ['ApiTestKey'];
            $apiUrl = self::$apiUrlPrefixTest . $apiUrl;
        }

        if ($apiKey) 
        {
            if ($objCurl = curl_init ()) 
            {
                curl_setopt_array ($objCurl, array (
                    CURLOPT_URL             => $apiUrl,
                    CURLOPT_HTTPAUTH        => CURLAUTH_BASIC,
                    CURLOPT_USERPWD         => $apiKey . ':',
                    CURLOPT_RETURNTRANSFER  => true,
                    CURLOPT_SSL_VERIFYPEER  => false,
                    CURLOPT_TIMEOUT         => 60,
                    CURLOPT_HTTPHEADER      => array (
                        'Content-Type: application/json'
                    )
                ));

                $result = curl_exec ($objCurl);
                
                if ($result === false) 
                {
                    $error = 'CURL error: ' . curl_error ($objCurl);

                    return array (
                        "Errors" => [ $error ]
                    );
                }
                
                curl_close ($objCurl);
            }

            return json_decode ($result, true);
        }

        return array ();
    }

    private static function apiPostRequest ($apiUrl, $params = [])
    {
        if (!(int) self::$settings ['TestMode'])
        {
            $apiKey = self::$settings ['ApiKey'];
            $apiUrl = self::$apiUrlPrefix.$apiUrl;
        }
        else
        {
            $apiKey = self::$settings ['ApiTestKey'];
            $apiUrl = self::$apiUrlPrefixTest.$apiUrl;
        }

        
        if ($apiKey) 
        {
            if ($objCurl = curl_init ()) 
            {
                curl_setopt_array ($objCurl, array (
                    CURLOPT_URL             => $apiUrl,
                    CURLOPT_HTTPAUTH        => CURLAUTH_BASIC,
                    CURLOPT_USERPWD         => $apiKey . ':',
                    CURLOPT_RETURNTRANSFER  => true,
                    CURLOPT_POST            => true,
                    CURLOPT_POSTFIELDS      => json_encode ($params),
                    CURLOPT_SSL_VERIFYPEER  => false,
                    CURLOPT_TIMEOUT         => 60,
                    CURLOPT_HTTPHEADER      => array (
                        'Content-Type: application/json'
                    )
                ));

                $result = curl_exec ($objCurl);

                if ($result === false) 
                {
                    $error = 'CURL error: ' . curl_error ($objCurl);

                    return array (
                        "Errors" => [ $error ]
                    );
                }

                curl_close ($objCurl);
            }

            return json_decode ($result, true);
        }

        return array ();
    }

    public static function getAffiliateData ($guid)
    {
        $affiliateData = [];
        $apiUrl = "v1/affiliate/";
        
        if ($guid) 
        {
            $apiUrl .= $guid;

            $affiliateData = self::apiGetRequest($apiUrl);
        }

        return $affiliateData;
    }

    public static function saveShippingMethod ($sm_id)
    {
        $_SESSION ['baikalsr__active_shipping'] = $sm_id;
    }

    public static function saveShippingDetails ($vars)
    {
        foreach ($vars as $name => $value)
        {
            $_SESSION ['baikalsr__shipping_data'][$name] = $value;
        }
    }

    public function getCityByName ($city, $region = "")
    {
        $cityData = [];
        $apiUrl = "v2/fias/cities?text=";
        if ($city) {
            $apiUrl .= $city;
            if ($region) $apiUrl .= " ".$region;

            $cityData = self::apiGetRequest($apiUrl);
        }
        
        return $cityData;
    }

    public function getCityByGuid ($guid)
    {
        $cityData = [];
        $apiUrl = "v2/fias/cities?guid=";
        if ($guid) {
            $apiUrl .= $guid;

            $cityData = self::apiGetRequest($apiUrl);
        }

        return $cityData;
    }

    public static function getStreetsByName ($cityGuid, $street = "")
    {
        $allStreetsData = [];
        $apiUrl = "v2/fias/streets?guid=".$cityGuid."&text=";
        if ($street) {
            $apiUrl .= $street;

            $allStreetsData = self::apiGetRequest($apiUrl);
        }

        return $allStreetsData;
    }

    public function getStreetByGuid ($guid)
    {
        $data = [];
        $apiUrl = "v2/fias/streets?streetguid=";

        if ($guid) 
        {
            $apiUrl .= $guid;

            $data = self::apiGetRequest($apiUrl);
        }

        return $data;
    }

    public static function getHousesByName($streetGuid, $house = "")
    {
        $allHousesData = [];

        $apiUrl = "v2/fias/objects?guid=".$streetGuid."&h=";

        if ($house) 
        {
            $apiUrl .= $house;
            $allHousesData = self::apiGetRequest($apiUrl);
        }

        return $allHousesData;
    }
    
    public function getHouseByGuid ($house_id)
    {
        list ($street_guid, $housenum, $buildnum, $strucnum) = explode ("|", $house_id);

        $data = [];
        $apiUrl = "v2/fias/objects?guid=$street_guid&h=$housenum&b=$buildnum&s=$strucnum";
        $data = self::apiGetRequest($apiUrl);

        return $data;
    }

    public static function getCargoTypes ()
    {
        $cargoTypes = [];
        $apiUrl = "v2/directory/cargotype";
        $cargoTypes = self::apiGetRequest($apiUrl);

        return $cargoTypes;
    }

    public static function getServices ($guid)
    {
        $services = [];
        if ($guid) {
            $apiUrl = "v2/directory/services/" . $guid;
            $services = self::apiGetRequest ($apiUrl);
        }

        return $services;
    }

    public function getPartnerTypes ()
    {
        $partnerTypes = [];
        $apiUrl = "v2/directory/partnertype";
        $partnerTypes = self::apiGetRequest($apiUrl);

        return $partnerTypes;
    }

    public static function getContragents ()
    {
        $contragents = [];
        $apiUrl = "v2/contragent/list";
        $contragents = self::apiPostRequest($apiUrl);

        return $contragents;
    }

	public static function sumSizeOneGoods ($xi, $yi, $zi, $qty)
    {
		$ar = array ($xi, $yi, $zi);
		
        sort ($ar);
		
        if ($qty <= 1) 
        {
            return (array ('X' => $ar [0], 'Y' => $ar [1], 'Z' => $ar [2]));
        }

		$x1 = 0;
		$y1 = 0;
		$z1 = 0;
		$l  = 0;

		$max1 = floor (sqrt ($qty));

		for ($y = 1; $y <= $max1; $y++) 
        {
			$i = ceil ($qty / $y);
		
            $max2 = floor (sqrt ($i));

			for ($z = 1; $z <= $max2; $z++) 
            {
				$x = ceil ($i / $z);

				$l2 = $x * $ar[0] + $y * $ar[1] + $z * $ar[2];
				
                if (($l == 0) || ($l2 < $l)) 
                {
					$l = $l2;
					$x1 = $x;
					$y1 = $y;
					$z1 = $z;
				}
			}
		}

		return array (
            'X' => $x1 * $ar [0], 
            'Y' => $y1 * $ar [1], 
            'Z' => $z1 * $ar [2]
        );
	}

	public static function sumSize ($a)
    {
		$n = count ($a);
		
        if (! ($n > 0)) 
        {
            return array (
                'length' => '0', 
                'width'  => '0', 
                'height' => '0'
            );
        }

		for ($i3 = 1; $i3 < $n; $i3++) 
        {
			for ($i2 = $i3 - 1; $i2 < $n; $i2++) 
            {
				for ($i = 0; $i <= 1; $i++) 
                {
					if ($a [$i2]['X'] < $a [$i2]['Y']) 
                    {
						$a1 = $a [$i2]['X'];
						$a [$i2]['X'] = $a [$i2]['Y'];
						$a [$i2]['Y'] = $a1;
					}

					if (($i == 0) && ($a [$i2]['Y'] < $a [$i2]['Z'])) 
                    {
						$a1 = $a [$i2]['Y'];
						$a [$i2]['Y'] = $a [$i2]['Z'];
						$a [$i2]['Z'] = $a1;
					}
				}

				$a [$i2]['Sum'] = $a [$i2]['X'] + $a [$i2]['Y'] + $a [$i2]['Z'];  
			}
            
			for ($i2 = $i3; $i2 < $n; $i2++)
            {
				for ($i = $i3; $i < $n; $i++)
                {
					if ($a [$i - 1]['Sum'] > $a [$i]['Sum']) 
                    {
						$a2 = $a [$i];
						$a [$i] = $a [$i - 1];
						$a [$i - 1] = $a2;
					}
                }
            }
                    
			if ($a [$i3 - 1]['X'] > $a [$i3]['X']) 
            {
                $a [$i3]['X'] = $a [$i3 - 1]['X'];
            }
            
            if ($a [$i3 - 1]['Y'] > $a [$i3]['Y']) 
            {
                $a [$i3]['Y'] = $a [$i3 - 1]['Y'];
            }
            
            $a [$i3]['Z']   = $a [$i3]['Z'] + $a [$i3 - 1]['Z'];
			$a [$i3]['Sum'] = $a [$i3]['X'] + $a [$i3]['Y'] + $a [$i3]['Z'];  
		}

		return array (
			'length' => round ($a [$n - 1]['X'], 2),
			'width'  => round ($a [$n - 1]['Y'], 2),
			'height' => round ($a [$n - 1]['Z'], 2)
		);
	}

    public function calculateOrder ($cartData, $calcType = null, $isDeliveryToTerminal = null) 
    {   
        $cityGuid = isset ($cartData ['cityGuid'])
            ? $cartData ['cityGuid']
            : $this->getCityByName($cartData ['city']) [0]['guid'];

        $apiUrl = "v2/calculator";

        list ($street_guid, $housenum, $buildnum, $strucnum) = explode ("|", self::$settings ["SenderHouse"]);

        $params = [
            "Departure"     => [ "CityGuid" => self::$settings ["SenderCity"] ],
            "Destination"   => [ "CityGuid" => $cityGuid ],
            "Cargo"         => [ "CargoList" => $cartData ['cargo_list'] ],
        ]; 
        
        if ($calcType === null) {
            $calcType = self::$settings ['CargoCalcType'];
        }
        
        foreach ($params ['Cargo']['CargoList'] as &$item)
        {
            if (empty ((float) $item ['Length']) || empty ((float) $item ['Width']) || empty ((float) $item ['Height']))
            {
                $item ['Length'] = self::$settings ['DefaultCargoLength'];
                $item ['Width']  = self::$settings ['DefaultCargoWidth'];
                $item ['Height'] = self::$settings ['DefaultCargoHeight'];

                $item ['defaultSizeInUse'] = true;
            }

            if (empty ((float) $item ['Weight']))
            {
                $item ['Weight'] = self::$settings ['DefaultCargoWeight'];

                $item ['defaultWeightInUse'] = true;
            }

            if (empty ($item ['Units']))
            {
                $item ['Units'] = 1;
            }
        }

        unset ($item);
  
        $services = json_decode (self::$settings ["AdditionalServices"], true);

        $servicesobject = [];

        foreach ($services as $id)
        {
            $count = 1;

            if ($id == 22) 
            {
                $count = self::$settings ["SmallBagCount"];
            }
        
            else if ($id == 23) 
            {
                $count = self::$settings ["MiddleBagCount"];
            }

            $servicesobject [] = [ 
                "id" => $id, "fields" => [[ "id" => $id, "value" => $count ]] 
            ];
        }

		
        foreach ($params ['Cargo']['CargoList'] as &$item)
        {
            if (!array_key_exists ('servicesobject', $item)) 
            {
                $item ['servicesobject'] = $servicesobject;
            }
        }

        unset ($item);
        
        if ($calcType == 'list') 
        {
            foreach ($params ['Cargo']['CargoList'] as &$item)
            {
                if ($item ['Units'] > 1)
                {
                    $result = self::sumSizeOneGoods (
                        $item ['Length'], $item ['Width'], $item ['Height'], 
                            ($item ['defaultSizeInUse'] && self::$settings ['CargoDefaultSettingsType'] == 'unit')
                                ? 1 
                                : $item ['Units']
                    );

                    $item ['Length'] = $result ['X'];
                    $item ['Width']  = $result ['Y'];
                    $item ['Height'] = $result ['Z'];

                    $item ['Volume'] = $result ['X'] * $result ['Y'] * $result ['Z'];

                    $item ['Weight'] = $item ['Weight'] * (
                        ($item ['defaultWeightInUse'] && self::$settings ['CargoDefaultSettingsType'] == 'unit') 
                            ? 1 
                            : $item ['Units']
                    );

                    $item ['Units']  = 1;
                }
            }
        }

        unset ($item);

        if ($calcType == 'summary')
        {
            $sumArray = [];

            $sumWeight = 0;     $sumCost   = 0;     $sumVolume = 0;

            foreach ($params ['Cargo']['CargoList'] as $item)
            {
                $units = ($item ['defaultSizeInUse'] && self::$settings ['CargoDefaultSettingsType'] == 'unit')
                    ? 1 
                    : $item ['Units'] ;

                for ($i = 0; $i < $units; $i++)
                {
                    $sumArray [] = [
                        'X' => $item ['Length'],
                        'Y' => $item ['Width'],
                        'Z' => $item ['Height']
                    ];
                }
            }

            foreach ($params ['Cargo']['CargoList'] as $item)
            {
                $sumWeight += $item ['Weight'] * (
                    ($item ['defaultWeightInUse'] && self::$settings ['CargoDefaultSettingsType'] == 'unit') 
                        ? 1 
                        : $item ['Units']
                );

                $sumCost   += $item ['EstimatedCost'];
            }

            $result = self::sumSize ($sumArray);

            $summary = $params ['Cargo']['CargoList'][0];

            $summary ['Length']         = $result ['length'];
            $summary ['Width']          = $result ['width'];
            $summary ['Height']         = $result ['height'];

            $summary ['Weight']         = $sumWeight;
            $summary ['Units']          = 1;

            $summary ['EstimatedCost']  = $sumCost;
            $summary ['Volume']         = $result ['length'] * $result ['width'] * $result ['height'];

            $params ['Cargo']['CargoList'] = [ $summary ];

            $params ['Cargo']['SummaryCargo'] = $params ['Cargo']['CargoList'][0];
    
            unset ($params ['Cargo']['CargoList']);
    
        }

        $isDepartureFromTerminal = self::$settings ["IsTerminal"];

        if ($isDeliveryToTerminal === null) {
            $isDeliveryToTerminal = @$_SESSION ['baikalsr__shipping_data']['is_terminal'];
        }

        if (!$isDepartureFromTerminal)
        {
            $params ["Departure"]["PickUp"] = [
                "Street"    => self::$settings ["SenderStreet"],
                "House"     => $housenum,
                "Housing"   => $buildnum,
                "Building"  => $strucnum,
                "TimeFrom"  => "09:00",
                "TimeTo"    => "18:00",
            ];

            if (self::$settings ["DefaultPRR"])
            {
                $params ["Departure"]["PickUp"]["Prr"] = [
                    "floor" => self::$settings ["DefaultFloor"],
                    "lift"  => self::$settings ["DefaultElevator"]
                ];
            }
        }
 
        if (!$isDeliveryToTerminal)
        {
            list ($street_guid, $housenum, $buildnum, $strucnum) = explode ("|", $cartData ["House"]);
            
            $params ["Destination"]["Delivery"] = [
                "Street"    => $street_guid,
                "House"     => $housenum,
                "Housing"   => $buildnum,
                "Building"  => $strucnum,
                "TimeFrom"  => "09:00",
                "TimeTo"    => "18:00", 
            ];

            if (isset ($cartData ["PRR"]))
            {
                $params ["Destination"]["Delivery"]["Prr"] = [
                    "floor" => $cartData ["Floor"],
                    "lift"  => $cartData ["Elevator"]
                ];
            }
        }

        $result = self::apiPostRequest($apiUrl, $params);

        $_SESSION ['baikalsr__shipping_data']['price'] = $result;

        $result ['DetailedInfo'] =  self::$settings ['DetailedInfo'];
        
        return $result;
    }

    public static function createOrder ($data)
    {
        $apiUrl = "v2/order";

        $result = self::apiPostRequest ($apiUrl, $data);

        if (!isset ($result ['errors']))
        {
            $request_id  = $result ['request_id'];
            $tracking_id = $result ['tracknumbers'][0];

            $db = JFactory::getDbo();
        
            $query = $db->getQuery (true);
    
			$query->update ($db->quoteName ('#__jshopping_baikalsr_orders'))
                  ->set ("track_number = " . $db->quote ("{$tracking_id}"))
                  ->set ("request_id = "   . $db->quote ("{$request_id}"))
                  ->set ("status = "       . $db->quote ("order_created"))
                  ->where ($db->quoteName ('order_id') . ' = ' . $db->quote ($data ['order_id']));

			$db->setQuery ($query);
            $db->execute ();

        }

        return $result;
    }

    public static function getOrderData ($orderId)
    {
        $orderData = [];
        
        if ($orderId) 
        {
            $apiUrl    = "v2/order/detail";
            $orderData = self::apiPostRequest ($apiUrl, ["id" => $orderId]);
        }

        return $orderData;
    }
    
    public static function getTrackingData ($track_id)
    {
        $result = [];
        
        if ($track_id) 
        {
            $apiUrl = "v1/tracking?number=" . $track_id;
            $result = self::apiGetRequest ($apiUrl);
        }

        return $result;
    }
}